(function( $, _, factory ) {

	var PsFocusArea = factory( $, _ );
	ps_focus_area = new PsFocusArea();

})( jQuery, _, function( $, _ ) {

function PsFocusArea() {
	$( $.proxy( this.init_page, this ) );
}

PsFocusArea.prototype = {

	init_page: function() {
		this.$focus = $('.ps-js-focus').eq( 0 );
		if ( ! this.$focus.length ) {
			return;
		}

		this.$focus_avatar = this.$focus.find('.ps-js-focus-avatar').on('click', $.proxy( this.onClickAvatar, this ));
		this.$focus_interactions = this.$focus.find('.ps-js-focus-interactions');
		this.$focus_links = this.$focus.find('.ps-js-focus-links');
		this.$focus_link_more = this.$focus.find('.ps-js-focus-link-more');
		this.$focus_link_dropdown = this.$focus_link_more.find('.ps-dropdown-menu');

		$( window ).on('resize.ps-focus-area', $.proxy( this.fix_focus_links_debounced, this ));
		this.fix_focus_links();
	},

	fix_focus_links_debounced: _.debounce(function() {
		this.fix_focus_links();
	}, 1000 ),

	fix_focus_links: function() {
		var counter = 0,
			isWide = ['medium', 'large'].indexOf( peepso.screenSize() ) >= 0,
			parentWidth, moveLast, $last;

		// reset links visibility
		this.$focus_link_dropdown.empty();
		this.$focus_link_more.hide();
		this.$focus_link_more.siblings(':hidden').css('display', '');

		parentWidth = this.$focus_links.parent().width();
		if ( isWide ) {
			parentWidth -= this.$focus_interactions.width();
		}

		do {
			// get last item
			$last = this.$focus_links.children('li:visible').last();
			moveLast = this.$focus_links.width() > parentWidth;

			// move last item if needed
			if ( moveLast ) {
				$last = this.$focus_link_more.siblings('li:visible').last();
				$last = $last.hide().clone();
				this.$focus_link_dropdown.prepend( $last.css('display', '') );
				if ( ! counter ) {
					this.$focus_link_more.css('display', '');
					this.$focus_link_dropdown.css('min-width', isWide ? '' : this.$focus_link_more.width() );
				}
			}

		// max 20 iterations to prevent potential measurement error due to external styling issue
		} while ( ++counter <= 20 && moveLast );
	},

	/**
	 * Show avatar dialog for user.
	 */
	showAvatarDialog: function() {
		var data;

		if ( peepsodata.profile ) {
			if ( ! this.avatarDialog ) {
				data = _.extend({}, peepsodata.profile );
				if ( ! data.hasAvatar ) {
					data.imgOriginal = '';
				}
				this.avatarDialog = new PsAvatarDialog( data );
			}
			this.avatarDialog.show();
		}
	},

	/**
	 * Handle click avatar on focus area.
	 * @param {HTMLEvent} e
	 */
	onClickAvatar: function( e ) {
		var handler;

		e.preventDefault();
		e.stopPropagation();

		handler = ps_observer.apply_filters('show_avatar_dialog', $.proxy( this.showAvatarDialog, this ));
		if ( _.isFunction( handler ) ) {
			handler();
		}
	}

};

return PsFocusArea;

});
