(function( $, peepso, factory ) {

	/**
	 * PsUtil global instance.
	 * @name peepso.util
	 * @type {PsUtil}
	 */
	peepso.util = new (factory( $ ));

})( jQuery, peepso || {}, function( $ ) {

/**
 * PeepSo utility class.
 * @class PsUtil
 */
function PsUtil() {}

peepso.npm.objectAssign( PsUtil.prototype, /** @lends PsUtil.prototype */ {

	/**
	 * Checks whether an element is fully visible in viewport.
	 * @param {HTMLElement} el
	 * @return {boolean}
	 */
	isElementInViewport: function( el ) {
		var rect = el.getBoundingClientRect();
		return (
			rect.top >= 0 &&
			rect.left >= 0 &&
			rect.bottom <= ( window.innerHeight || document.documentElement.clientHeight ) &&
			rect.right <= ( window.innerWidth || document.documentElement.clientWidth )
		);
	},

	/**
	 * Checks whether an element is partly visible in viewport.
	 * @param {HTMLElement} el
	 * @return {boolean}
	 */
	isElementPartlyInViewport: function( el ) {
		var rect = el.getBoundingClientRect();
		return (
			rect.top < ( window.innerHeight || document.documentElement.clientHeight ) &&
			rect.left < ( window.innerWidth || document.documentElement.clientWidth ) &&
			rect.bottom > 0 &&
			rect.right > 0
		);
	},

	/**
	 * Scroll element into top of the viewport.
	 * @param {HTMLElement} el
	 * @param {number} [duration=1000]
	 */
	scrollIntoView: function( el, duration ) {
		el = $( el );
		if ( el.length && ! this.isElementInViewport( el[0] ) ) {
			$('html, body').animate({
				scrollTop: Math.max( 0, el.offset().top - 10 )
			}, duration || 1000 );
		}
	},

	/**
	 * Scroll element into top of the viewport if it is not already visible.
	 * @param {HTMLElement} el
	 * @param {number} [duration=1000]
	 */
	scrollIntoViewIfNeeded: function( el, duration ) {
		el = $( el );
		if ( el.length && ! this.isElementPartlyInViewport( el[0] ) ) {
			$('html, body').animate({
				scrollTop: Math.max( 0, el.offset().top - 10 )
			}, duration || 1000 );
		}
	}

});

return PsUtil;

});
